import copy
import ctypes
import os
import platform
import subprocess

import astropy.units as u
import dask
import matplotlib.patches as patches
import matplotlib.pyplot as plt
import numpy as np
import threading
from dask.distributed import Client
import json



import pygsfit_cp.pygsfit_cp_utils as ut
from pygsfit_cp import ndfits
import h5py


class pygsfit_cp:
    def __init__(self, filename=None, out_dir=None, fit_fov=None):
        self.filename = filename if filename is not None else os.path.join(os.path.dirname(__file__),
                                                                           'demo/eovsa_allbd_demo_0.fits')
        #self.filename = filename
        self.out_dir = out_dir if out_dir else os.path.join(os.path.expanduser('~'), 'pygsfit_cp_output')
        if not os.path.exists(self.out_dir):
            os.makedirs(self.out_dir)

        self.libpath = self.get_lib()
        self.margin_pix = 5
        self.meta, self.tb_data = ndfits.read(self.filename)
        cx1 = self.meta['header']['CRVAL1'] - self.meta['header']['CDELT1']*  self.meta['header']['CRPIX1']*0.5
        cx2 = self.meta['header']['CRVAL1'] + self.meta['header']['CDELT1']*  self.meta['header']['CRPIX1']*0.5
        cy1 = self.meta['header']['CRVAL2'] - self.meta['header']['CDELT2'] * self.meta['header']['CRPIX2'] * 0.5
        cy2 = self.meta['header']['CRVAL2'] + self.meta['header']['CDELT2'] * self.meta['header']['CRPIX2'] * 0.5
        self.fit_fov = fit_fov if fit_fov is not None else [[cx1,cy1],[cx2,cy2]] # in format of fov [[x1, y1],[x2,y2]]
        self.start_freq = self.meta['ref_cfreqs'][0]  # Hz
        self.end_freq = self.meta['ref_cfreqs'][-1]  # Hz
        self.freq_ghz = self.meta['ref_cfreqs'] / 1e9
        self.integrated_threshold_sfu = 1  # sfu
        self.rms_factor = 1  # Uncertainty_rms = RMS * rms_factor
        self.background_xyrange = [[0.15, 0.75], [0.25,
                                                  0.85]]  # percentage of the start and end range in X and Y, should be within [0,1]
        self.pix_mask=None
        self.ninput = np.array([7, 0, 1, 30, 1, 1],
                               dtype='int32')  # Nparms, Angular_mode,Npix, Nfreq(will be replaced) fitting mode, stokes
        self.rinput = np.array([0.17, 1e-6, 1.0, 4.0, 8.0, 0.015], dtype='float64')  # real_input
        self.initial_parguess = np.array([
            [10.0, 0.0001, 2000.0],  # n_nth;    1d7 cm^-3
            [4.0, 0.01, 30.0],  # B;    1d2G
            [60.0, 22.0, 87.0],  # theta;    deg
            [10.0, 0.01, 600.0],  # n_th;    1d9 cm^-3
            [4.5, 1.6, 10.0],  # Delta;    No
            [5.0, 0.1, 10.0],  # E_max;    MeV
            [5.0, 1.5, 60.0],  # T_e;    MK
        ], dtype=np.float64)
        if self.meta['header']['BTYPE'] == 'Brightness Temperature' or 'K' in self.meta['header']['BUNIT']:
            self.get_3d_fluxdata_array()
        self.plot_mode = True
        self.update_flux_threshold_mask()
        self.update_rms()

    def get_lib(self):
        if platform.system() == 'Linux' or platform.system() == 'Darwin':
            unix_dir = os.path.join(os.path.dirname(__file__), 'unix/')
            libpath = os.path.join(unix_dir, 'fit_Spectrum_Kl.so')
            if platform.machine() == 'arm64':
                unix_dir = os.path.join(os.path.dirname(__file__), 'unix/arm64/')
                libpath = os.path.join(unix_dir, 'fit_Spectrum_Kl.so')
        if platform.system() == 'Windows':
            libpath = os.path.join(os.path.dirname(__file__), 'win/gs_fit_1D.dll')

        if not os.path.exists(libpath):
            makefile = os.path.join(unix_dir, 'makefile')
            if makefile:
                #cwd = os.path.dirname(__file__)
                os.chdir(unix_dir)
                # subprocess.run(['rm', '*.o'])
                subprocess.run(['make', 'clean'])
                subprocess.run(['make'])
                if not os.path.exists(libpath):
                    raise Exception('The attempt to make a shareable library failed, not able to go on....')
        return libpath

    def get_3d_fluxdata_array(self):
        # ndfits file usually in Tb, if not, skip this step
        flx_data = np.zeros_like(self.tb_data)
        for freq_idx, cfreq in enumerate(self.meta['ref_cfreqs']):
            flx_data[0, freq_idx, :, :] = ut.sfu2tb_2d(cfreq, self.tb_data[0, freq_idx, :, :],
                                                       area=self.meta['header']['CDELT1'] * self.meta['header'][
                                                           'CDELT2'] * u.arcsec ** 2, reverse=True)
        self.flux_data = flx_data[0, :, :, :]

    def do_fit(self, mode='batch', inp_coord=None, world=True):
        """

        :param mode: 'single' or 'batch'
        :param inp_coord: [930, -260]  (world, when world==True) or [124,200] (pixel, when world == False), always in x, y order.
        :param world:
        :return:
        """
        self.update_flux_threshold_mask()
        start_freq_idx = np.argmin(np.abs(self.meta['ref_cfreqs'] - self.start_freq))
        end_freq_idx = np.argmin(np.abs(self.meta['ref_cfreqs'] - self.end_freq))+1
        self.ffghz = np.array(self.meta['ref_cfreqs'][start_freq_idx:end_freq_idx] / 1.e9, dtype='float64')
        self.ninput[3] = len(self.ffghz)
        spec_in_list = []
        if mode == 'single':
            if inp_coord is not None:
                if world:
                    cur_pix = ut.world2pix(inp_coord, self.meta['refmap'])
                    inp_coord = [int(cur_pix[0].value), int(cur_pix[1].value)]
                self.coordinates = [inp_coord[::-1]]
            else:
                print('The first pixel of the selected ROI will be fitted')
        else:
            self.plot_mode=False
        for coord_idx, coord in enumerate(self.coordinates):
            spec_in = np.zeros((1, end_freq_idx - start_freq_idx, 4), dtype='float64', order='F')
            spec_in[0, :, 0] = self.flux_data[start_freq_idx:end_freq_idx, coord[0], coord[1]]
            spec_in[0, :, 2] = self.rms[start_freq_idx:end_freq_idx] + self.rms_factor * self.rms[
                                                                                         start_freq_idx:end_freq_idx] / self.ffghz
            spec_in_list.append(spec_in)
        print('Current fitting range is {0} - {1} Hz'.format(self.start_freq, self.end_freq))
        print('Long inputs are: ', self.ninput)
        print('Real inputs are: ', self.rinput)
        print('Params(ranges) are: ', self.initial_parguess)
        print('{0} pixels to be fitted'.format(len(self.coordinates)))
        cfmode = "plotting (single pixel)" if self.plot_mode else "batch"
        print(f"will be fitted in {cfmode} mode")
        #user_response = input("Do you want to continue? Enter 'y' for Yes or 'n' for No: ").strip().lower()
        #while True:
        #if user_response == 'y':
        #print("Continuing...")
        repeated_row = np.array([5.0, 0.2, 20.0], dtype=np.float64)
        repeated_rows = np.tile(repeated_row, (8, 1))
        final_parguess = np.asfortranarray(np.vstack((self.initial_parguess, repeated_rows)))
        if mode == 'single':
            fit_res = pyWrapper_Fit_Spectrum_Kl(self.libpath, self.ninput, self.rinput, final_parguess,
                                                self.ffghz, spec_in_list[0])
            plot_fitting_res(fit_res, self.ffghz, spec_in_list[0])
            return 1
        # elif mode == 'batch':
        #     tasks = []
        #     for cidx, cur_spec_in in enumerate(spec_in_list):
        #         extra_info = {'filename': self.filename, 'start_freq_idx': start_freq_idx,
        #                       'end_freq_idx': end_freq_idx,
        #                       'parguess': final_parguess, 'ninput': self.ninput, 'rinput': self.rinput,
        #                       'coord': list(self.coordinates[cidx]),
        #                       'spec_in': spec_in_list[cidx], 'freq_fitted': self.ffghz, 'task_idx': cidx,
        #                       'out_dir': self.out_dir, 'rms_range': self.background_xyrange, 'data_saved_range': self.data_saved_range,
        #                       'world_center': self.data_saved_world_center}
        #
        #         # Number of parallel tasks
        #         cspec_in = copy.deepcopy((spec_in_list[cidx]))
        #
        #         # tasks.append(dask.delayed(pyWrapper_Fit_Spectrum_Kl)(self.libpath, self.ninput, self.rinput,
        #         #                                                      final_parguess, self.ffghz,
        #         #                                                      spec_in_list[cidx], info=extra_info))
        #         tasks.append(dask.delayed(pyWrapper_Fit_Spectrum_Kl)(copy.deepcopy(self.libpath), copy.deepcopy(self.ninput), copy.deepcopy(self.rinput), copy.deepcopy(final_parguess),
        #                              copy.deepcopy(self.ffghz), cspec_in, info=copy.deepcopy(extra_info)))
        elif mode == 'batch':
            client = Client(processes=True, n_workers=4)
            tasks = []
            for cidx, cur_spec_in in enumerate(spec_in_list):
                extra_info = {'filename': self.filename, 'start_freq_idx': start_freq_idx,
                              'end_freq_idx': end_freq_idx,
                              'parguess': final_parguess, 'ninput': self.ninput, 'rinput': self.rinput,
                              'coord': list(self.coordinates[cidx]),
                              'spec_in': spec_in_list[cidx], 'freq_fitted': self.ffghz, 'task_idx': cidx,
                              'out_dir': self.out_dir, 'rms_range': self.background_xyrange,
                              'data_saved_range': self.data_saved_range,
                              'world_center': self.data_saved_world_center}

                cspec_in = copy.deepcopy((spec_in_list[cidx]))

                task = dask.delayed(pyWrapper_Fit_Spectrum_Kl)(copy.deepcopy(self.libpath),
                                                               copy.deepcopy(self.ninput),
                                                               copy.deepcopy(self.rinput),
                                                               copy.deepcopy(final_parguess),
                                                               copy.deepcopy(self.ffghz), cspec_in,
                                                               info=copy.deepcopy(extra_info))
                tasks.append(task)

            # Compute tasks using dask
            out_filenames = dask.compute(*tasks, scheduler='processes')
            #out_filenames = dask.compute(*tasks)
            final_out_fname = os.path.join(self.out_dir,os.path.basename(self.filename).replace('fits', 'h5'))
            merge_task = dask.delayed(ut.combine_hdf5_files)(out_filenames, final_out_fname, True)
            merge_task.compute()
            return final_out_fname
        else:
            print('Mode can only be single or batch')

        #break  # Break out of the loop to continue execution within the function
            # elif user_response == 'n':
            #     print("Exiting function...")
            #     return  # Exit the current function
            # else:
            #     print("Invalid input. Please enter 'y' for Yes or 'n' for No.")

    def update_flux_threshold_mask(self):
        """
            if self.pix_mask is provided, fit_fov would not be used
            if self.integrated_threshold_sfu<=0.0, would not be used
        """
        if self.pix_mask is None:
            self.fov_mask = ut.create_fov_mask(self.meta['refmap'], self.fit_fov)
        else:
            self.fov_mask = self.pix_mask
        fov_y, fov_x = np.where(~self.fov_mask)
        self.fov_coordinates = list(zip(fov_y, fov_x))
        self.final_mask = self.fov_mask
        if self.integrated_threshold_sfu>=0.0:
            if not hasattr(self, 'rms'):
                self.update_rms()
            # creat data mask in X and Y plane with provided threshold(in sfu)
            # squeezed_flxdata = (data - rms[:, np.newaxis, np.newaxis]) * freq_ghz[:, np.newaxis, np.newaxis]
            squeezed_flxdata = self.flux_data - self.rms[:, np.newaxis, np.newaxis]
            summed_over_freq = squeezed_flxdata.sum(axis=0)
            masked_data = np.ma.masked_where(summed_over_freq <= self.integrated_threshold_sfu, summed_over_freq)
            self.threshold_mask = masked_data.mask
            if np.all(self.threshold_mask):
                print('The threshold is too large, none of the pixels is selected. Please try again')
                return
            self.final_mask = np.logical_or(self.fov_mask, self.threshold_mask)
        y, x = np.where(~self.final_mask)
        self.coordinates = list(zip(y, x))
        y_values = [coord[0] for coord in self.fov_coordinates]
        x_values = [coord[1] for coord in self.fov_coordinates]
        self.min_y = max(min(y_values)-self.margin_pix, 0)
        self.max_y = min(max(y_values)+self.margin_pix, self.meta['refmap'].data.shape[0])
        self.min_x = max(min(x_values)-self.margin_pix, 0)
        self.max_x = min(max(x_values)+self.margin_pix, self.meta['refmap'].data.shape[1])
        self.data_saved_range = [[self.min_y,self.max_y], [self.min_x, self.max_x]]
        world_center = self.meta['refmap'].pixel_to_world(int((self.min_x+self.max_x)/2)*u.pix,int((self.min_y+self.max_y)/2)*u.pix )
        self.data_saved_world_center = [world_center.Tx.value, world_center.Ty.value]

    def update_rms(self, xyrange=None):
        """

        :param xyrange: xyrange: percentage of the start and end range in X and Y, should be within [0,1]
        """
        if xyrange is not None:
            self.background_xyrange = xyrange
        [[xstart, ystart], [xend, yend]] = self.background_xyrange
        height, width = self.flux_data[0].shape
        self.rms = np.zeros((len(self.freq_ghz)), dtype=np.float64)
        for i in range(len(self.freq_ghz)):
            self.rms[i] = np.std(self.flux_data[i,
                                 int(height * ystart):int(height * yend),
                                 int(width * xstart):int(width * xend)]) * self.rms_factor

    def plot_threshold_mask_rms(self, tar_freq_ghz=None):
        """
        Show the current mask and background region used to calculate the RMS.
        :param tar_freq_ghz: The freq to be displayed in GHz
        """
        if tar_freq_ghz is not None:
            target_idx = np.argmin(np.abs(self.meta['ref_cfreqs'] - tar_freq_ghz * 1.e9))
        else:
            target_idx = np.argmin(np.abs(self.meta['ref_cfreqs'] - self.start_freq))

        # plot the eovsa image at selected freq in GHz
        extent = ut.extent_convertor(self.meta['header'])
        img = plt.imshow(self.flux_data[target_idx, :, :], origin='lower', extent=extent, cmap='jet')
        img_m = plt.imshow(~self.final_mask, origin='lower', extent=extent, cmap='viridis',
                           alpha=0.5)  # plot the mask based on the threshold

        # Plot a box showing the region where RMS is calculated
        box_percentage = self.background_xyrange
        x_range = extent[1] - extent[0]  # Total x range in solar coordinates
        y_range = extent[3] - extent[2]  # Total y range in solar coordinates
        box_origin_solar = (extent[0] + box_percentage[0][0] * x_range, extent[2] + box_percentage[0][1] * y_range)
        box_width_solar = (box_percentage[1][0] - box_percentage[0][0]) * x_range
        box_height_solar = (box_percentage[1][1] - box_percentage[0][1]) * y_range
        plt.gca().add_patch(
            patches.Rectangle(box_origin_solar, box_width_solar, box_height_solar, linewidth=1, edgecolor='r',
                              facecolor='none'))
        plt.gca().add_patch(
            patches.Rectangle((self.fit_fov[0][0], self.fit_fov[0][1]), self.fit_fov[1][0]-self.fit_fov[0][0], self.fit_fov[1][1]-self.fit_fov[0][1], linewidth=1, edgecolor='g',
                              facecolor='none'))

        plt.gca().set_xlabel('Solar X \n [arcsec]')
        plt.gca().set_ylabel('Solar Y \n [arcsec]')
        plt.title('{0} pixels are selected'.format(np.sum(~self.final_mask)))
        rms_position = (box_origin_solar[0] + 0.02 * box_width_solar, box_origin_solar[1] + 0.02 * box_height_solar)
        fov_position = (self.fit_fov[0][0], self.fit_fov[0][1]-10)
        plt.text(*rms_position, 'Background\n Region', color='r', fontsize=9, va='bottom')
        plt.text(*fov_position, 'FOV', color='g', fontsize=9, va='bottom')
        cbar = plt.colorbar(img)
        cbar.set_label('Flux Density [sfu]')

        # img.set_extent(extent)
        plt.show()

    def documentation(self, inp_name):
        """
        Calling convention of each inp arr
        :param inp_name:only 'ninput', 'rinput', and 'initial_parguess' are supported
        """
        file_paths = {'ninput': './docs/Long_input.txt', 'rinput': './docs/Real_input.txt',
                      'initial_parguess': './docs/Parms_input.txt'}
        with open(file_paths[inp_name], 'r') as file:
            content = file.read()
            print("=============")
            print(content)
            print("=============")


    def save_notebook_cells(self, notebook_path, pyfile_path):
        """
        Save code cells from a Jupyter notebook to a Python script file in a main() format.

        :param notebook_path: Path to the input Jupyter notebook file.
        :param pyfile_path: Path to the output Python script file.
        """
        to_script_tag = 'to_script tag'
        init_tag = '#init multiple\n'

        # Load the notebook
        with open(notebook_path, 'r') as nbfile:
            notebook = json.load(nbfile)

        # Open the output Python file
        with open(pyfile_path, 'w') as pyfile:
            #Write the imports
            code_block_import = """
                from pygsfit_cp.pygsfit_cp import *
                import os
                import pkg_resources
                from pygsfit_cp.pygsfit_cp_utils import *
                import pygsfit_cp.results_convertor as rc
            """
            indented_source = '\n'.join(line.lstrip() for line in code_block_import.splitlines())
            pyfile.write(indented_source + '\n\n')
            # Write the main function header
            pyfile.write('def main():\n')

            # If multiple_frames is True, find and write the '#init multiple' block first
            #if multiple_frames:
            for cell in notebook['cells']:
                if cell['cell_type'] == 'code':
                    if init_tag in cell['source']:
                        # Add the '#init multiple' block indented for the main() function
                        indented_source = ''.join('    ' + line for line in cell['source'])
                        pyfile.write(indented_source + '\n\n')
                        break
            for cell in notebook['cells']:
                if cell['cell_type'] == 'code':
                    cell_source = ''.join(cell['source'])
                    if to_script_tag in cell_source:
                        indented_source = ''.join('        ' + line for line in cell['source'])
                        pyfile.write(indented_source + '\n\n')
            code_block_1 = """
            cur_map_file = rc.create_params_map(cur_save_file)
            map_files_by_frame.append(cur_map_file)
            fitting_files_by_frame.append(cur_save_file)
            """
            indented_source = '\n'.join('        ' + line.lstrip() for line in code_block_1.splitlines())
            pyfile.write(indented_source + '\n\n')
            code_block_2 = """
            fitting_res_file = os.path.join(gs.out_dir,'fitting_results.h5')
            combine_hdf5_files(file_list=fitting_files_by_frame,output_file=fitting_res_file) #please make sure the original files would NOT be deleted!
            print('All the fitting results are save to ', fitting_res_file)
            print('The map files are: ' )
            print(map_files_by_frame)
            rh_pro = os.path.join(package_path,'IDL_utils','read_hdf5.pro')
            pp_pro = os.path.join(package_path,'IDL_utils','process_params_maps.pro')
            mp_pro = os.path.join(gs.out_dir,'make_params_maps.pro')
            rc.create_idl_pro_file(rh_pro, pp_pro, map_files_by_frame, mp_pro)
            print('Fitting and priliminary processed are finished, please run {} in IDL to create the sav file for gsviewer')
            """
            indented_source = '\n'.join('    ' + line.lstrip() for line in code_block_2.splitlines())
            pyfile.write(indented_source + '\n\n')

            # Write the main execution block
            pyfile.write("\n\nif __name__ == '__main__':\n")
            pyfile.write("    main()\n")

        print(pyfile_path, 'is saved.')


lock = threading.Lock()
def pyWrapper_Fit_Spectrum_Kl(cur_libpath, ninput, rinput, parguess, freq, spec_in, info=None):
    """
    A python wrapper to call  Dr.Fleishman's Fortran code: fit_Spectrum_Kl.for/fit_Spectrum_Kl.so, all the input should be
    numpy array with dtype='float64', order='F', txt files can be created by calling get_Table().
    :param ninput: np.array([7, 0, 1, 30, 1, 1], dtype='int32'), 30 here will be replace by n_freq later. See Long_input.txt
    :param rinput: see real_input.txt, for example: np.array([0.17, 1e-6, 1.0, 4.0, 8.0, 0.015], dtype='float64')
    :param parguess: Input parameters ([guess, min, max]*15), see Parms_input.txt
    :param freq: freqs in GHz, example:    freq = np.array([3.42, 3.92, 4.42,.......], dtype='float64', order='F')
    :param spec_in: spectrum/uncertainty to be fitted, (1, n_freq, 4), spectrum:[0,:,0], uncertainty:[0,:,2]
    :param info: extra info to be save, if provided, fitting results will be saved in to a hdf5.
    :return:fitted spectrum, parameters and corresponding uncerntainties.
    """
    # orgnize the input args
    n_freq = len(freq)
    ninput[3] = np.int32(n_freq)
    # create the output holder
    spec_out = np.zeros((1, n_freq, 2), dtype='float64', order='F')
    aparms = np.zeros((1, 8), dtype='float64', order='F')
    eparms = np.zeros((1, 8), dtype='float64', order='F')

    # prepare the pointer of the i/o array
    inp_arrays = [ninput, rinput, parguess, freq, spec_in, aparms, eparms, spec_out]
    ct_pointers = [arr.ctypes.data_as(ctypes.POINTER(ctypes.c_double)) for arr in inp_arrays]
    ninput_ct, rinput_ct, parguess_ct, freq_ct, spec_in_ct, aparms_ct, eparms_ct, spec_out_ct = ct_pointers
    argv = (ctypes.POINTER(ctypes.c_double) * 8)(ninput_ct, rinput_ct, parguess_ct, freq_ct, spec_in_ct, aparms_ct,
                                                 eparms_ct, spec_out_ct)
    with lock:
        if platform.system() == 'Linux' or platform.system() == 'Darwin':
            libc_mw = ctypes.CDLL(cur_libpath)
            mwfunc = libc_mw.get_mw_fit_
        if platform.system() == 'Windows':
            libc_mw = ctypes.WinDLL(cur_libpath)
            mwfunc = libc_mw.get_mw_fit
        res = mwfunc(ctypes.c_longlong(8), argv)
    if info is not None:
        out_fname = os.path.join(info['out_dir'], 'task_{0:0=4d}.hdf5'.format(info['task_idx']))
        #info_serialized = json.dumps(info)
        with h5py.File(out_fname, 'w') as f:
            # Create datasets within the HDF5 file
            f.create_dataset('spec_out', data=spec_out)
            f.create_dataset('aparms', data=aparms)
            f.create_dataset('eparms', data=eparms)
            info_group = f.create_group('info')
            for key, value in info.items():
                #if isinstance(value, (int, float, str, list, np.ndarray)):
                if isinstance(value, list):
                    value = np.array(value)
                info_group.create_dataset(key, data=value)
                #else:
                #    raise TypeError(f"Unsupported data type for key {key}: {type(value)}")
            #f.create_dataset('info', data=info)
        # with h5py.File('yourfile.h5', 'r') as f:
        #     info = json.loads(f['info'][()].decode())
        #with open(out_fname, 'wb') as f:
        #    pickle.dump((spec_out, aparms, eparms, info), f)
        # pickle.dump((spec_out, aparms, eparms, info), open(out_fname, 'wb'), protocol=pickle.HIGHEST_PROTOCOL)
        return out_fname
    else:
        return (spec_out, aparms, eparms, info)


def plot_fitting_res(fit_res, freq, spec_fitted):
    plt.plot(freq, fit_res[0][0, :, 0] + fit_res[0][0, :, 1], label='model')
    plt.errorbar(freq, spec_fitted[0, :, 0], yerr=spec_fitted[0, :, 2], linestyle='None', marker='*', label='data')
    ax = plt.gca()
    ax.set_yscale('log')
    ax.set_xscale('log')
    ax.set_ylabel('Flux Density [sfu]')
    ax.set_xlabel('Freq [Hz]')
    ax.legend()
    # print('fitting result are: ', fit_res[1])
    plt.show()



from astropy import constants as const, units as u
import numpy as np
from astropy.coordinates import SkyCoord
import h5py
import os
import zipfile
import tarfile
from astropy.io import fits
from pygsfit_cp import ndfits

def extent_convertor(cheader):
    # Replace direct assignments with lookups
    CRVAL1 = cheader['CRVAL1']
    CDELT1 = cheader['CDELT1']
    CRPIX1 = cheader['CRPIX1']
    NAXIS1 = cheader['NAXIS1']

    CRVAL2 = cheader['CRVAL2']  # Assuming similar keys for Y-axis
    CDELT2 = cheader['CDELT2']
    CRPIX2 = cheader['CRPIX2']
    NAXIS2 = cheader['NAXIS2']

    # Calculate extent
    x_extent_left = CRVAL1 - (CRPIX1 - 1) * CDELT1
    x_extent_right = x_extent_left + NAXIS1 * CDELT1
    y_extent_bottom = CRVAL2 - (CRPIX2 - 1) * CDELT2
    y_extent_top = y_extent_bottom + NAXIS2 * CDELT2
    extent = [x_extent_left, x_extent_right, y_extent_bottom, y_extent_top]
    return extent



def sfu2tb_2d(frequency, flux_2d, area=None, square=True, reverse=False, verbose=False):
    """
    Convert a 2D array of flux densities (in sfu) to brightness temperatures (in K), or vice versa,
    across a given frequency, considering the source's area or assuming a square/elliptical shape.

    Parameters:
        frequency: Frequency in Hz (assumed same for the entire 2D array)
        flux_2d: 2D array of flux densities in sfu (if not reverse) or brightness temperatures in K (if reverse)
        area: Source area in arcsec^2 (if known)
        square: Assume square-shaped source if True; elliptical if False (ignored if area is specified)
        reverse: Convert brightness temperature to flux density if True
        verbose: Print detailed operations if True
    """

    c = const.c.cgs
    k_B = const.k_B.cgs
    sfu = u.Jy * 1e4  # sfu to Jansky conversion

    # Frequency handling
    frequency = frequency * u.Hz if not hasattr(frequency, 'unit') else frequency

    # Flux handling
    flux_unit = u.K if reverse else sfu
    flux_2d = flux_2d * flux_unit if not hasattr(flux_2d, 'unit') else flux_2d

    if area is None:
        raise ValueError('Area must be provided or calculable from source size.')

    area = area * (u.arcsec ** 2) if not hasattr(area, 'unit') else area

    sr = area.to(u.radian ** 2)
    factor = c ** 2 / (2 * k_B * frequency ** 2 * sr)

    if reverse:
        if verbose: print('Converting input brightness temperatures in K to flux densities in sfu for each point.')
        result = (flux_2d / factor).to(sfu, equivalencies=u.dimensionless_angles())
    else:
        if verbose: print('Converting input flux densities in sfu to brightness temperatures in K for each point.')
        result = (flux_2d * factor).to(u.K, equivalencies=u.dimensionless_angles())

    return result


def world2pix(xy_coord, refmap):
    solar_x = xy_coord[0] * u.arcsec
    solar_y = xy_coord[1] * u.arcsec
    coord = SkyCoord(solar_x, solar_y, frame=refmap.coordinate_frame)
    return refmap.world_to_pixel(coord)

def merge_hdf5(files, output_file):
    with h5py.File(output_file, 'w') as h5f:
        for i, file in enumerate(files):
            with h5py.File(file, 'r') as f:
                for key in f.keys():
                    data = f[key][:]
                    if key in h5f:
                        h5f[key].resize((h5f[key].shape[0] + data.shape[0]), axis=0)
                        h5f[key][-data.shape[0]:] = data
                    else:
                        maxshape = (None,) + data.shape[1:]
                        h5f.create_dataset(key, data=data, maxshape=maxshape, chunks=True)

def copy_items(source_group, target_group):
    for item_name, item in source_group.items():
        if isinstance(item, h5py.Dataset):
            source_group.copy(item, target_group, name=item_name)
        elif isinstance(item, h5py.Group):
            new_group = target_group.create_group(item_name)
            copy_items(item, new_group)

def combine_hdf5_files(file_list, output_file, delete_original=False):
    with h5py.File(output_file, 'w') as new_hdf:
        for file_path in file_list:
            group_name = os.path.splitext(os.path.basename(file_path))[0]
            group = new_hdf.create_group(group_name)
            with h5py.File(file_path, 'r') as source_hdf:
                copy_items(source_hdf, group)
    if delete_original:
        for file_path in file_list:
            os.remove(file_path)
            print(f"Deleted {file_path}")

def create_fov_mask(sunpy_map, fov):
    """
    Create a boolean mask for the specified FOV on a SunPy map.

    Parameters:
    sunpy_map (sunpy.map.GenericMap): The SunPy map.
    fov (list): The field of view in the format [[x_bl, y_bl], [x_tr, y_tr]] in arcseconds.

    Returns:
    numpy.ndarray: A boolean mask with the same dimensions as the SunPy map.
    """
    # Convert FOV world coordinates to pixel coordinates
    bottom_left = sunpy_map.world_to_pixel(SkyCoord(fov[0][0]*u.arcsec, fov[0][1]*u.arcsec, frame=sunpy_map.coordinate_frame))
    top_right = sunpy_map.world_to_pixel(SkyCoord(fov[1][0]*u.arcsec, fov[1][1]*u.arcsec, frame=sunpy_map.coordinate_frame))
    #top_right = sunpy_map.world_to_pixel(fov[1][0]*u.arcsec, fov[1][1]*u.arcsec)

    # Extract the pixel coordinates as integers
    x_bl, y_bl = int(bottom_left.x.value), int(bottom_left.y.value)
    x_tr, y_tr = int(top_right.x.value), int(top_right.y.value)

    # Ensure the coordinates are within the map boundaries
    x_bl = max(0, x_bl)
    y_bl = max(0, y_bl)
    x_tr = min(sunpy_map.data.shape[1], x_tr)
    y_tr = min(sunpy_map.data.shape[0], y_tr)

    # Create the mask
    mask = np.zeros(sunpy_map.data.shape, dtype=bool)
    mask[y_bl:y_tr, x_bl:x_tr] = True

    return ~mask

def makelist(tdir='', keyword1='', keyword2='', exclude=None):
    li = []
    # for root, dirs, files in os.walk(tdir):
    root = os.getcwd()
    files = os.listdir(tdir)
    for file in files:
        if exclude is None:
            if keyword1 in file and keyword2 in file:
                li.append(os.path.join(tdir, file))
        else:
            if keyword1 in file and keyword2 in file and exclude not in file:
                li.append(os.path.join(tdir, file))

    return li


def process_input_path(input_path):
    """
    Processes the input path. If the input is a folder, returns all .fits files in the folder.
    If the input is a compressed file (zip, tar, etc.), it extracts the file into the same directory
    and returns all .fits files in the extracted folder.

    :param input_path: Path to a directory or a compressed file.
    :return: List of .fits files in the directory.
    """
    if is_valid_fits_file(input_path):
        return [input_path]
    # Check if input path is a directory
    if os.path.isdir(input_path):
        # If it's a directory, return the list of .fits files in this directory
        fits_files = [os.path.join(input_path, f) for f in os.listdir(input_path) if f.endswith('.fits')]
        return fits_files

    # If the input is not a directory, check if it's a compressed file
    elif os.path.isfile(input_path):
        file_dir = os.path.dirname(input_path)
        extracted_folder_name = os.path.splitext(os.path.basename(input_path))[0]
        extracted_folder_path = os.path.join(file_dir, extracted_folder_name)

        # If it's a .zip file
        if input_path.endswith('.zip'):
            with zipfile.ZipFile(input_path, 'r') as zip_ref:
                zip_ref.extractall(extracted_folder_path)

        # If it's a .tar, .tar.gz, or .tar.bz2 file
        elif input_path.endswith(('.tar', '.tar.gz', '.tar.bz2')):
            with tarfile.open(input_path, 'r') as tar_ref:
                tar_ref.extractall(extracted_folder_path)
        else:
            raise ValueError("Unsupported file format. Please provide a directory or a supported compressed file (.zip, .tar, .tar.gz, .tar.bz2).")

        # After extracting, return the list of .fits files in the extracted folder
        fits_files = []
        for root, dirs, files in os.walk(extracted_folder_path):
            for file in files:
                if file.endswith('.fits'):
                    fits_files.append(os.path.join(root, file))
        return fits_files

    else:
        raise ValueError("Input path is neither a directory nor a supported compressed file.")

def is_valid_fits_file(filepath):
    try:
        with fits.open(filepath) as hdul:
            # If the file opens successfully, it's a FITS file
            return True
    except Exception:
        # If there's an error opening the file, it's not a FITS file
        return False

def get_fixed_fov_mask(inp_file, inp_fov):
    cmeta, ctb_data = ndfits.read(inp_file)
    cur_fixed_mask = create_fov_mask(cmeta['refmap'], inp_fov)
    return cur_fixed_mask





#== A python convertor allows user to convert the the fitting reslults in the bach mode to a hdf5 file which can be
#== converted to a sav file that can be opened by the the gsfitviewer

from scipy.io import readsav
import h5py
from pygsfit_cp import ndfits
from datetime import datetime
import numpy as np
import os


def read_sample():
    demo_sav = '/Users/walterwei/ssw/packages/gsfit/demo/demo_eovsa_gsfitmaps.sav'
    demo = readsav(demo_sav,python_dict=True)
    print('')


def create_template_group(f, cmeta, info):
    #cmeta,tb_data = ndfits.read('/Users/walterwei/Downloads/20220511/slf_final_XX_t19_allbd.fits')
    #eparms = 1.0
    #with h5py.File(tmp_fname, 'w') as f:
    template_group = f.create_group('TEMPLATE')
    template_group.create_dataset('B0', data=0.0)#????
    num_xaxis = info['data_saved_range'][1][1]-info['data_saved_range'][1][0]
    num_yaxis = info['data_saved_range'][0][1]-info['data_saved_range'][0][0]
    #template_group.create_dataset('DATA', data=np.zeros(cmeta['refmap'].meta['naxis1'], cmeta['refmap'].meta['naxis2']))
    template_group.create_dataset('DATA', data=np.zeros(num_yaxis, num_xaxis))
    template_group.create_dataset('DATAUNITS', '')
    template_group.create_dataset('DUR', data=cmeta['refmap'].meta['exptime'])
    template_group.create_dataset('DX', data=cmeta['refmap'].meta['cdelt1'])
    template_group.create_dataset('DY', data=cmeta['refmap'].meta['cdelt2'])
    template_group.create_dataset('FREQ', data=cmeta['refmap'].meta['crval3']/1.e9)
    template_group.create_dataset('FREQUNIT', data='GHz')
    template_group.create_dataset('ID', '')
    template_group.create_dataset('L0', data=0.0)#????
    template_group.create_dataset('RMS', data=0.0) #?????
    template_group.create_dataset('RMSUNITS', data='sfu')
    template_group.create_dataset('RMSXRAN', data=[info['rms_range'][0][0], info['rms_range'][1][0]])
    template_group.create_dataset('RMSYRAN', data=[info['rms_range'][0][1], info['rms_range'][1][1]])
    template_group.create_dataset('ROLL_ANGLE', data=0.00) #????
    template_group.create_dataset('ROLL_CENTER', data=[0, 0])
    template_group.create_dataset('RSUN', data=cmeta['refmap'].meta['rsun_obs'])
    template_group.create_dataset('SCALEFAC', data=0.90299976) #????
    template_group.create_dataset('SNR', data=100.0) #????
    template_group.create_dataset('SOHO', data=0)#????
    template_group.create_dataset('STOKES', data=cmeta['pol_names'][0])
    parsed_date = datetime.strptime(cmeta['refmap'].meta['date-obs'], '%Y-%m-%dT%H:%M:%S.%f')
    template_group.create_dataset('TIME', data=parsed_date.strftime('%d-%b-%Y %H:%M:%S.%f')[:-3])
    template_group.create_dataset('XC', data=info['world_center'][0])
    template_group.create_dataset('XUNITS', data=cmeta['refmap'].meta['cunit1'])
    template_group.create_dataset('YC', data=info['world_center'][1])
    template_group.create_dataset('YUNITS', data=cmeta['refmap'].meta['cunit2'])

def copy_and_modify_group(src_group, dst_group, new_params_dict):
    for name, dataset in src_group.items():
        data = dataset[()]
        if name in new_params_dict:
            data = new_params_dict[name]
        dst_group.create_dataset(name, data=data)

def create_params_map(save_file, map_file=None):
    """
    convert the output of the saving files into parameter maps, usage can be found in
    Tutorial.ipynb
    @param save_file: the output of the spectral fitting (h5 file) :
    @param map_file:  the output
    """
    base, ext = os.path.splitext(save_file)
    if map_file is None:
        map_file = f"{base}_params_map{ext}"
    print('map file will be saved to ', map_file)
    cinfo = read_info(save_file)
    cmeta, tb_data = ndfits.read(cinfo['filename'])
    res_array, peak_flux, peak_freq,wb, wb_err, wnth, wnth_err , flux_data, err_data, fit_data = read_fitting_result(save_file, cmeta, tb_data, cinfo)
    sig_array = [peak_flux, peak_freq,wb, wb_err, wnth, wnth_err]
    sig_array = [carray[cinfo['data_saved_range'][0][0]:cinfo['data_saved_range'][0][1], cinfo['data_saved_range'][1][0]:cinfo['data_saved_range'][1][1]] for carray in sig_array]
    res_array = [carray[cinfo['data_saved_range'][0][0]:cinfo['data_saved_range'][0][1], cinfo['data_saved_range'][1][0]:cinfo['data_saved_range'][1][1]] for carray in res_array]
    peak_flux, peak_freq, wb, wb_err, wnth, wnth_err = sig_array
    dat_array = [flux_data, err_data, fit_data]
    dat_array = [carray[:,:,cinfo['data_saved_range'][0][0]:cinfo['data_saved_range'][0][1],
                 cinfo['data_saved_range'][1][0]:cinfo['data_saved_range'][1][1]] for carray in dat_array]
    flux_data, err_data, fit_data = dat_array
    #res_array = cur_array[:, cinfo['data_saved_range'][0][0]:cinfo['data_saved_range'][0][1], cinfo['data_saved_range'][1][0]:cinfo['data_saved_range'][1][1]]
    params_dict = {
        'B': {'DATAUNITS': 'G', 'ID': 'B', 'DATA':res_array[1]},
        'CHISQR': {'DATAUNITS': '', 'ID': 'CHISQR', 'DATA':res_array[7]},
        'DELTA': {'DATAUNITS': '', 'ID': 'Delta', 'DATA':res_array[4]},
        'ERRB': {'DATAUNITS': 'G', 'ID': 'errB', 'DATA':res_array[9]},
        'ERRDELTA': {'DATAUNITS': '', 'ID': 'errDelta', 'DATA':res_array[12]},
        'ERRE_MAX': {'DATAUNITS': 'MeV', 'ID': 'errE_max', 'DATA':res_array[13]},
        'ERRN_NTH': {'DATAUNITS': 'cm^-3', 'ID': 'errn_nth', 'DATA':res_array[8]},
        'ERRN_TH': {'DATAUNITS': 'cm^-3', 'ID': 'errn_th', 'DATA':res_array[11]},
        'ERRPEAKFLUX': {'DATAUNITS': 'sfu', 'ID': 'errpeakflux', 'DATA':np.zeros_like(peak_flux)}, #?????
        'ERRPEAKFREQ': {'DATAUNITS': 'GHz', 'ID': 'errpeakfreq', 'DATA':np.zeros_like(peak_freq)}, #?????
        'ERRTHETA': {'DATAUNITS': 'deg', 'ID': 'errtheta', 'DATA':res_array[10]},
        'ERRT_E': {'DATAUNITS': 'MK', 'ID': 'errT_e', 'DATA':res_array[14]},
        'ERRWB': {'DATAUNITS': 'erg', 'ID': 'errWB', 'DATA':wb_err},
        'ERRWNTH': {'DATAUNITS': 'erg', 'ID': 'errWnth', 'DATA':wnth_err}, #????? #keV
        'E_MAX': {'DATAUNITS': 'MeV', 'ID': 'E_max', 'DATA':res_array[5]},
        'N_NTH': {'DATAUNITS': 'cm^-3', 'ID': 'n_nth', 'DATA':res_array[0]},
        'N_TH': {'DATAUNITS': 'cm^-3', 'ID': 'n_th', 'DATA':res_array[3]},
        'PEAKFLUX': {'DATAUNITS': 'sfu', 'ID': 'peakflux', 'DATA':peak_flux},
        'PEAKFREQ': {'DATAUNITS': 'GHz', 'ID': 'errpeakfreq', 'DATA':peak_freq},
        'RESIDUAL': {'DATAUNITS': 'sfu', 'ID': 'Residual', 'DATA':np.zeros_like(res_array[7])}, #?????????
        'THETA': {'DATAUNITS': 'deg', 'ID': 'theta', 'DATA': res_array[2]},
        'T_E': {'DATAUNITS': 'MK', 'ID': 'T_e', 'DATA': res_array[6]},
        'WB': {'DATAUNITS': 'erg', 'ID': 'WB', 'DATA': wb},
        'WNTH': {'DATAUNITS': 'erg', 'ID': 'Wnth', 'DATA': wnth},  # ????? #keV

    }
    with h5py.File(map_file, 'w') as f:
        maps_group = f.create_group('MAPS')
        create_template_group(f, cmeta, cinfo)
        template_group = f['TEMPLATE']

        for cparm in params_dict.keys():
            small_group = maps_group.create_group(cparm)

            new_params = {
                'ID': params_dict[cparm]['ID'],
                'DATAUNITS': params_dict[cparm]['DATAUNITS'],
                'DATA': params_dict[cparm]['DATA']
            }
            copy_and_modify_group(template_group, small_group, new_params)

        freq_dependent_terms = ['DATAMAPS', 'ERRMAPS', 'FITMAPS']
        for cki, ckey in enumerate(freq_dependent_terms):
            maps_group.create_group(ckey)
        for cfi, cfreq in enumerate(cmeta['ref_cfreqs']/1.e9):
            spw_group = maps_group['DATAMAPS'].create_group('spw_{0:0=2d}'.format(cfi))
            fdt_params = {'DATAUNITS': 'sfu', 'ID': 'Measured Flux @ {} GHz'.format(cfreq), 'DATA':flux_data[0,cfi,:,:], 'FREQ': cfreq}
            copy_and_modify_group(template_group, spw_group, fdt_params)

            spw_group = maps_group['ERRMAPS'].create_group('spw_{0:0=2d}'.format(cfi))
            fdt_params = {'DATAUNITS': 'sfu', 'ID': 'Applied Fit Error @ {} GHz'.format(cfreq), 'DATA':err_data[0,cfi,:,:], 'FREQ': cfreq}
            copy_and_modify_group(template_group, spw_group, fdt_params)

            spw_group = maps_group['FITMAPS'].create_group('spw_{0:0=2d}'.format(cfi))
            fdt_params = {'DATAUNITS': 'sfu', 'ID': 'Fit Flux @ {} GHz'.format(cfreq), 'DATA':fit_data[0,cfi,:,:], 'FREQ': cfreq}
            copy_and_modify_group(template_group, spw_group, fdt_params)
        del f['TEMPLATE']
    print('results are saved to ', map_file)
    return map_file

def read_fitting_result(save_file, cmeta, tb_data, cinfo):
    with h5py.File(save_file, 'r') as f:
        f_arrays = [np.zeros((cmeta['refmap'].meta['naxis1'], cmeta['refmap'].meta['naxis2'])) for _ in range(16)]
        peak_flux = np.zeros((cmeta['refmap'].meta['naxis1'], cmeta['refmap'].meta['naxis2']))
        peak_freq = np.zeros((cmeta['refmap'].meta['naxis1'], cmeta['refmap'].meta['naxis2']))
        wb = np.zeros((cmeta['refmap'].meta['naxis1'], cmeta['refmap'].meta['naxis2']))
        wb_err = np.zeros((cmeta['refmap'].meta['naxis1'], cmeta['refmap'].meta['naxis2']))
        wnth = np.zeros((cmeta['refmap'].meta['naxis1'], cmeta['refmap'].meta['naxis2']))
        wnth_err = np.zeros((cmeta['refmap'].meta['naxis1'], cmeta['refmap'].meta['naxis2']))
        vol_pix = cinfo['rinput'][3]*cinfo['rinput'][4]*(7.27e7**3) #arcsec^3 area*depth
        flux_data =np.zeros_like(tb_data)
        err_data = np.zeros_like(tb_data)
        fit_data = np.zeros_like(tb_data)
        # for spwi in range(tb_data.shape[1]):
        #     flux_data[0,spwi,:,:] = sfu2tb_2d(cinfo['freq'][spwi]*1.e9,tb_data[0,spwi,:,:],area=cmeta['refmap'].meta['cdelt1']*cmeta['refmap'].meta['cdelt2']* u.arcsec ** 2,reverse=True)
        task_names = [name for name in f.keys() if name.startswith('task_')]
        for task_name in task_names:
            group_cont = read_group(f[task_name])
            # if task_name == task_names[0]:
            #     cinfo = group_cont['info']

            aparms = group_cont['aparms'][0]
            eparms = group_cont['eparms'][0]

            coord = group_cont['info']['coord']
            y, x = coord
            for i, value in enumerate(group_cont['info']['spec_in'][0, :, 0]):
                flux_data[0,i+cinfo['start_freq_idx'], y, x] = value
                err_data[0,i+cinfo['start_freq_idx'],  y, x] = group_cont['info']['spec_in'][0, i, 2]
                fit_data[0,i+cinfo['start_freq_idx'],  y, x] = group_cont['spec_out'][0, i, 0]+group_cont['spec_out'][0, i, 1]
            for i, value in enumerate(aparms):
                f_arrays[i][y, x] = value
            for i, value in enumerate(eparms, start=len(aparms)):
                f_arrays[i][y, x] = value

            peak_flux[y,x] = np.max(flux_data[0,:,y, x])
            peak_freq[y,x] = cmeta['ref_cfreqs'][np.argmax(flux_data[0,:,y, x])]/1.e9
            wb[y,x] = vol_pix*aparms[1]**2/(8.0*np.pi)
            wb_err[y,x] = vol_pix*2*aparms[1]*eparms[1]/(8.0*np.pi)
            wnth[y,x] = vol_pix*1.6e-9*cinfo['rinput'][5]*1.e3*(aparms[4]-1)/(aparms[4]-2)*aparms[0]#? #convert emin from MeV to keV?
            wnth_err[y,x] = vol_pix*1.6e-9*cinfo['rinput'][5]*1.e3*(eparms[4]**2*eparms[0]**2/(aparms[4]-2)**4+((aparms[4]-1)/(aparms[4]-2)*eparms[0])**2)**0.5 #convert emin from MeV to keV
    return (f_arrays, peak_flux, peak_flux, wb, wb_err, wnth, wnth_err, flux_data, err_data, fit_data)

def read_info(save_file):
    with h5py.File(save_file, 'r') as f:
        task_names = [name for name in f.keys() if name.startswith('task_')]
        group_cont = read_group(f[task_names[0]])
        cinfo = group_cont['info']
    return cinfo

def read_group(group):
    content = {}
    for key, item in group.items():
        if isinstance(item, h5py.Dataset):
            content[key] = item[()]
        elif isinstance(item, h5py.Group):
            content[key] = read_group(item)
    return content

def create_idl_pro_file(path1, path2, file_list, output_pro_file):
    """
    Creates an IDL .pro file with the provided paths and file list.

    Parameters:
    path1 (str): The first file path to include in the .pro file.
    path2 (str): The second file path to include in the .pro file.
    file_list (str): The variable name or content to be used in the process_files call.
    output_pro_file (str): The output .pro file name to write.
    """
    # Construct the content of the .pro file
    content = f"""@{path1}
    @{path2}
    file_list = {file_list}
    process_files, file_list
    ;process_files, file_list, '/path/to/your_own_sav_file.sav'
    end
    """

    # Write the content to the specified output .pro file
    with open(output_pro_file, 'w') as file:
        file.write(content)

    print(f"{output_pro_file} has been created successfully.")


def main():
    import pygsfit_cp_utils as ut
    res_list = ut.makelist(tdir='/Users/walterwei/Downloads/20220511/gsfit_test/pix_t/', keyword1='allbd.h5')
    for i, cres in enumerate(res_list):
       create_params_map(cres)




if __name__ == "__main__":
    main()


from setuptools import setup, find_packages
#from numpy.distutils.core import Extension, setup
import platform
#from setuptools import setup, Extension
#from numpy import get_include
#from numpy.f2py import f2py_extension

# fflags = ['-ffixed-line-length-0', '-march=native', '-O3', '-fPIC']
# if platform.system() == 'Darwin':  # macOS
#     fflags.append('-static-libgfortran')
#
# fortran_extension = Extension(
#     name='pygsfit.fit_Spectrum_Kl',
#     sources=[
#         'pygsfit_cp/fortran_src/Calc_GS_Spec_hom.for',
#         'pygsfit_cp/fortran_src/angular.for',
#         'pygsfit_cp/fortran_src/spidrsub.for',
#         'pygsfit_cp/fortran_src/FitFun.for',
#         'pygsfit_cp/fortran_src/ResolvedSpectrum.for',
#         'pygsfit_cp/fortran_src/klein.f',
#         'pygsfit_cp/fortran_src/fit_Spectrum_Kl.for'
#     ],
#     extra_f90_compile_args=fflags,
#     extra_f77_compile_args=fflags,
#     extra_link_args=['-shared'],
# )

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='pygsfit_cp',  #
    version='0.1.0',  #
    description='A python wrpper of the core fitting functionalities of gsFit.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Yuqian Wei',
    author_email='yw633@njit.edu',
    packages=find_packages(),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.8',
    #ext_modules=[fortran_extension],  #
    install_requires=[
        'numpy>=1.25.0',  #
        'astropy>=6.0.0',
        'dask>=2024.3.0',
        'h5py>=3.6.0',
        'matplotlib>=3.8.2',
        'scipy>=1.12.0',
        'sunpy>=5.1.1',
    ],
    setup_requires=[
        'numpy',  #
    ],
)